#include <stdio.h>
#include <vector>
#include <windows.h>
#include <winnls.h>
#include "tjs.h"
#include "tjsError.h"
#include "tjsByteCodeLoader.h"

#include "tjsNative.h"
class tTJSNC_Debug : public tTJSNativeClass {
public:
	tTJSNC_Debug();
	static tjs_uint32 ClassID;
protected:
	tTJSNativeInstance * CreateNativeInstance();
};
//---------------------------------------------------------------------------
// tTJSNC_Debug
//---------------------------------------------------------------------------
tjs_uint32 tTJSNC_Debug::ClassID = -1;
tTJSNC_Debug::tTJSNC_Debug() : tTJSNativeClass(TJS_W("Debug"))
{
	TJS_BEGIN_NATIVE_MEMBERS(Debug)
	TJS_DECL_EMPTY_FINALIZE_METHOD
//----------------------------------------------------------------------
TJS_BEGIN_NATIVE_CONSTRUCTOR_DECL_NO_INSTANCE(/*TJS class name*/Debug)
{
	return TJS_S_OK;
}
TJS_END_NATIVE_CONSTRUCTOR_DECL(/*TJS class name*/Debug)
//----------------------------------------------------------------------
//-- methods
//----------------------------------------------------------------------
TJS_BEGIN_NATIVE_METHOD_DECL(/*func. name*/message)
{
	if(numparams<1) return TJS_E_BADPARAMCOUNT;

	if(numparams == 1) {
		wprintf( ((const ttstr&)*param[0]).c_str() );
	} else {
		// display the arguments separated with ", "
		ttstr args;
		for(int i = 0; i<numparams; i++) {
			if(i != 0) args += TJS_W(", ");
			args += ttstr(*param[i]);
		}
		wprintf( args.c_str() );
	}
	wprintf( L"\n" );
	return TJS_S_OK;
}
TJS_END_NATIVE_STATIC_METHOD_DECL(/*func. name*/message)
//----------------------------------------------------------------------
	TJS_END_NATIVE_MEMBERS
} // end of tTJSNC_Debug::tTJSNC_Debug
tTJSNativeInstance *tTJSNC_Debug::CreateNativeInstance() { return NULL; }
tTJSNativeClass * TVPCreateNativeClass_Debug() { return new tTJSNC_Debug(); }


#define REGISTER_OBJECT(classname, instance) \
	dsp = (instance); \
	val = tTJSVariant(dsp/*, dsp*/); \
	dsp->Release(); \
	global->PropSet(TJS_MEMBERENSURE|TJS_IGNOREPROP, TJS_W(#classname), NULL, \
		&val, global);

class OuputClass : public iTJSConsoleOutput {
public:
	void ExceptionPrint(const tjs_char *msg) {
		wprintf( msg );
		wprintf( L"\n" );
	}
	void Print(const tjs_char *msg) {
		wprintf( msg );
		wprintf( L"\n" );
	}
};
class OutputStream : public tTJSBinaryStream {
	FILE* FilePointer;

public:
	OutputStream( const char* path ) {
		FilePointer = fopen( path, "wb" );
	}
	bool IsOpend() const {
		return FilePointer != NULL;
	}
	//-- must implement
	tjs_uint64 TJS_INTF_METHOD Seek(tjs_int64 offset, tjs_int whence) {
		if( !FilePointer ) TJS_eTJSScriptError( TJSInternalError, (tTJSScriptBlock*)NULL, 0 );
		return fseek( FilePointer, offset, whence );
	}

	//-- optionally to implement
	tjs_uint TJS_INTF_METHOD Read(void *buffer, tjs_uint read_size) { return 0; }
		/* returns actually read size */

	tjs_uint TJS_INTF_METHOD Write(const void *buffer, tjs_uint write_size) {
		if( !FilePointer ) TJS_eTJSScriptError( TJSInternalError, (tTJSScriptBlock*)NULL, 0 );
		return fwrite( buffer, 1, write_size, FilePointer );
	}

	//-- should re-implement for higher performance
	tjs_uint64 TJS_INTF_METHOD GetSize() {
		if( !FilePointer ) TJS_eTJSScriptError( TJSInternalError, (tTJSScriptBlock*)NULL, 0 );
		long pos = ftell( FilePointer );
		fseek( FilePointer, 0, SEEK_END );
		long size = ftell( FilePointer );
		fseek( FilePointer, pos, SEEK_SET );
		return size;
	}

	virtual ~OutputStream() {
		if( FilePointer ) fclose( FilePointer );
	}
};

int main( int argc, char* argv[] ) {
	setlocale(LC_ALL, ""); // P[ݒ
	tTJS *tjsengine = new tTJS(); // TJS2 XNvgGW쐬
	OuputClass console;
	tjsengine->SetConsoleOutput( &console );
	try {
		tTJSVariant val;
		iTJSDispatch2 *dsp;
		iTJSDispatch2 *global = tjsengine->GetGlobalNoAddRef();

		/* classes */
		REGISTER_OBJECT(Debug, TVPCreateNativeClass_Debug());
#if 0
		if( argc > 1 ) {
			FILE* fp = fopen(argv[1], "rb" );
			if( fp ) {
				fseek(fp,  0L, SEEK_END);
				size_t size = ftell(fp);
				char* buff = new char[size];
				fseek(fp,  0L, SEEK_SET);
				size_t readlen = fread( buff, 1, size, fp );
				fclose(fp);
				
				if( tTJSByteCodeLoader::IsTJS2ByteCode( reinterpret_cast<const tjs_uint8*>(buff) ) ) {
					tTJSVariant result; // ʂ󂯎邽߂̕ϐ
					tjsengine->LoadByteCode( reinterpret_cast<tjs_uint8*>(buff), size, &result, NULL, TJS_W("test code") );
				} else {
					int	len = ::MultiByteToWideChar( CP_ACP, 0, buff, readlen, NULL, 0);
					std::vector<wchar_t> outbuf( len+1, 0 );
					int	ret = ::MultiByteToWideChar( CP_ACP, 0, buff, readlen, &(outbuf[0]), len );
					if( ret ) {
						outbuf[ret] = L'\0';
						if( argc > 2 ) {
							OutputStream* stream = new OutputStream( argv[2] );
							tjsengine->CompileScript( &(outbuf[0]), stream, false, TJS_W("test code"), 0 );
							delete stream;
						} else {
							tTJSVariant result; // ʂ󂯎邽߂̕ϐ
							tjsengine->ExecScript( &(outbuf[0]), &result, NULL, TJS_W("test code"));    // XNvgs
						}
					}
				}
			}
		}
#else
		if( argc > 2 ) {
			FILE* fp = fopen(argv[1], "rb" );
			if( fp ) {
				fseek(fp,  0L, SEEK_END);
				size_t size = ftell(fp);
				char* buff = new char[size];
				fseek(fp,  0L, SEEK_SET);
				size_t readlen = fread( buff, 1, size, fp );
				fclose(fp);
				
				int	len = ::MultiByteToWideChar( CP_ACP, 0, buff, readlen, NULL, 0);
				std::vector<wchar_t> outbuf( len+1, 0 );
				int	ret = ::MultiByteToWideChar( CP_ACP, 0, buff, readlen, &(outbuf[0]), len );
				if( ret ) {
					outbuf[ret] = L'\0';
					OutputStream* stream = new OutputStream( argv[2] );
					if( stream->IsOpend() ) {
						tjsengine->CompileScript( &(outbuf[0]), stream, false, false, false, TJS_W("test code"), 0 );
						delete stream;
					} else {
						printf( "Cannot open output file.\n" );
					}
				}
			} else {
				printf( "Cannot open input file.\n" );
			}
		} else {
			printf( "ctjs2 [input tjs2 script] [output tjs2 bytecode]\n" );
		}
#endif
		/*
		tTJSVariant result; // ʂ󂯎邽߂̕ϐ
		tjsengine->ExecScript(
			TJS_W( "function test(x, y) { return x*y; } \nreturn test(4, 5);\n"),
			&result, NULL,
			TJS_W("test code"));    // eXgXNvgs
			*/
	} catch(eTJSError &e) {
		printf("G[܂ : %ls\n", e.GetMessage().c_str());
	} catch(...) {
		printf("G[܂\n");
	}
	tjsengine->Shutdown(); // TJS2 XNvgGWVbg_E
	tjsengine->Release(); // TJS2 XNvgGW
	return 0;
}
